	module rfactor
!	=================================================
!	Module contains functions related to rfactor
!   and some general stand alone routines 
!   ==================================================
!	General array sizes
!	==================================================
	integer  maxang,maxnel,maxnat,iin
	parameter(maxang = 400, maxnat = 1000,maxnel=6,iin=20)
	contains
	subroutine meis_rfactor(meis,sim_name,exp_name,	&
				skew,skewopt,nelem,		&
				incy,weighty,n_angles,	&
				calopt,calib,calerr)
! ===================================================
!	AUTHOR : P.QUINN
!	FUNCTION TO EVALUATE MEIS R-FACTOR
!   Follows igor pro 
!	STEPS AS OUTLINED BY P.BAILEY
! ===================================================
! Declare local variables
	 implicit none
     INTEGER    ::   I,N_ANGLES,n_expangs,n_thangs,K,nelem	
	 integer    ::   incy(nelem),calopt,skewopt
	 real		::   ff,exp_step,th_step,meis,count
	 REAL       ::   THTMP(maxnel)
	 real		::	 calib,calerr
	 real		::   weighty(nelem)
	 REAL	    ::   YE(maxang),YTH(maxang),EXPANG(maxang),THANG(maxang) 
	 REAL  	    ::   SUMTH,SUMEXP,LAMDA,PI
	 REAL	    ::   degtorad,calup,callow
	 CHARACTER(80):: EXP_NAME,SIM_NAME,SKEW


! CALCULATE PI AND DEG TO RADIANS CONVERSION
	pi = 4.0*atan(1.0)
	degtorad=pi/180.0


!	READ IN EXP AND THEORY VALUES
	open(unit=iin,file=exp_name,status='old')	
	do i=1,maxang
	    read(iin,*,end=5) expang(i),ye(i)
	end do 
5       continue
	close(iin)
	n_expangs = i-1
	exp_step=expang(2)-expang(1)


!	read in angle and total yield
	OPEN(UNIT=iin,FILE=sim_name,STATUS='OLD')
        do i =1,maxang
            read(iin,*,end=10) thang(i),(thtmp(k),k=1,nelem)
		yth(i)=0.0
			DO K=1,NELEM
				IF(INCY(K).EQ.1) THEN
					YTH(i)=YTH(i)+THTMP(K)*WEIGHTY(K)
				END IF
			END DO
	end do
10	continue

	CLOSE(iin)	

        N_THANGS=i-1
	th_step=thang(2)-thang(1)



! IF YOU USED A STEP SIZE OTHER THAN 0.2 DEGREES THEN INTERPOLATE SIMULATION TO
! TO MATCH EXPERIMENT IF NEEDED.
	if(th_step.ne.exp_step) then 
		CALL INTERPOLATE(th_step,exp_step,thang,yth,n_expangs,n_thangs)
	endif
!
!	find region over which angles overlap
!	call overlap(thang,expang,ei_exp,ef_exp,ei_th,ef_th)	


	N_ANGLES=N_THANGS


! 3 POINT SMOOTH (MOVING AVERAGE) THE SIM DATA TWICE
	
     DO I=2,N_ANGLES-1
        YTH(I)=(YTH(I-1)+YTH(I)+YTH(I+1))/3.0
     END DO
  	 


! CALCULATE LAMDA THE RATIO OF THE AVERAGE VALUES OF THE
! EXP AND SIM DATA

	SUMTH =0.0
	SUMEXP=0.0

	DO I=1,N_ANGLES
	  SUMTH=SUMTH+YTH(I)
	  SUMEXP=SUMEXP+YE(I)
	END DO
	LAMDA=SUMEXP/SUMTH  	
! IF USING CALIBRATION CONSTRAIN TO BOUNDS
	IF(CALOPT.EQ.1) THEN
		CALUP = CALIB*(100.0+CALERR)/100.0
		CALLOW= CALIB*(100.0-CALERR)/100.0
		IF(LAMDA.LT.CALLOW) THEN
			LAMDA=CALLOW
		ELSE IF(LAMDA.GT.CALUP) THEN
			LAMDA=CALUP
		END IF 
	END IF
	
	DO I=1,N_ANGLES
	   ye(i)=ye(i)/lamda	   	
	END DO	


	call skewquad(YE,YTH,EXPANG,N_ANGLES)
! this scales yexp and ysim to counts
	DO I=1,N_ANGLES
	   ye(I)=ye(I)*LAMDA
	   ytH(I)=yth(I)*LAMDA
	END DO	



	open(unit=iin,file=skew,status='unknown')
	DO I=1,N_ANGLES 
		 write(iin,'(1X,F7.3,1X,F8.3,1X,F8.3,1X,F8.3,1X,F8.3)')    &
					expang(i),ye(i),yth(i),(ye(i)/lamda),(yth(i)/lamda)
		  END DO	 
	close(iin)	


! Apply Rutherford cross section to both data sets
	call rcs(ye,expang,n_angles)
	call rcs(ytH,expang,n_angles)



! MEIS R-FACTOR

	ff=0.0
	DO I=1,N_ANGLES
	   ff=ff+((YE(I)-YTH(I))**2)/YE(I)
	END DO	

	ff=ff/N_ANGLES	
	meis=ff	

	RETURN
	end subroutine meis_rfactor


	SUBROUTINE INTERPOLATE(th_step,exp_step,ANGLE,YT, &
     			n_expangs,n_thangs)
! ============================================================
!      The experimental data comes in 0.2 angular step size
!      To speed up the simulation process you can use larger
!      steps in the simulations and then interpolate to a 0.2 step
!      size. 
!      This subroutine achieves this by using a cubic spline
!      subroutine taken from the ever trust worthy 
!	 Numerical Recipies in Fortran 77
! ============================================================
	implicit none
	INTEGER n_expangs,n_thangs,newang,i
	REAL exp_step,th_step,YT(maxang),ANGLE(maxang),TANGLE(maxang),h
	real yt2(maxang),yp1,ypn,ang1, fang,newy(maxang)

	h=th_step

!	 Calculation of start and end points
!    which are derivatives at those points
!

    yp1=(2.*yt(4)-9.*yt(3)+18.*yt(2)-11.*yt(1))/(6.*H)
	ypn=(11.*yt(n_thangs)-18.*yt(n_thangs-1)+9.*yt(n_thangs-2) &
		-2.*yt(n_thangs-3))/(6.*H) 	



	call spline(angle,yt,n_thangs,yp1,ypn,yt2)

! New angular range	
	newang=((angle(n_thangs)-angle(1))/exp_step)+1

! CHOOSE SMALLEST UPPER ANGLES
	if(n_expangs.gt.newang) then
 	  n_expangs=newang
	else
	  newang=n_expangs
	end if


	do i=1,newang
	 ang1=((i-1)*exp_step)+angle(1)
	 tangle(i)=ang1 
	 call splint(angle,yt,yt2,n_thangs,ang1,fang)
	 newy(i)=fang
	end do

	do i=1,newang	
	 angle(i)=tangle(i)
	 yt(i)=newy(i)	
	end do
	n_thangs=newang

	end subroutine interpolate 	


     	SUBROUTINE spline(x,y,n,yp1,ypn,y2)
!============================================
! Taken from Numerical Recipies in F77
!============================================
    	INTEGER n,NMAX 
     	INTEGER i,k 
     	REAL yp1,ypn,x(n),y(n),y2(n) 
     	PARAMETER (NMAX=500) 
     	REAL p,qn,sig,un,u(NMAX) 
     	if (yp1.gt..99e30) then 
     	y2(1)=0. 
     	u(1)=0. 
    	else 
     	y2(1)=-0.5 
     	u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1) 
    	endif 
     	do 11 i=2,n-1 
     	sig=(x(i)-x(i-1))/(x(i+1)-x(i-1)) 
     	p=sig*y2(i-1)+2. 
     	y2(i)=(sig-1.)/p 
     	u(i)=(6.*((y(i+1)-y(i))/(x(i+  &
     1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig* & 
     u(i-1))/p 
11     continue 
       if (ypn.gt..99e30) then 
        qn=0. 
        un=0. 
       else 
         qn=0.5 
         un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1))) 
       endif 
       y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.) 
       do 12 k=n-1,1,-1 
        y2(k)=y2(k)*y2(k+1)+u(k) 
12     continue 
       return 
       END subroutine spline



      SUBROUTINE splint(xa,ya,y2a,n,x,y) 
! ============================================
! Taken from Numerical recipies in F77
! ============================================
      INTEGER n 
      INTEGER k,khi,klo 
      REAL x,y,xa(n),y2a(n),ya(n) 
      REAL a,b,h 
      klo=1 
      khi=n 
1     if (khi-klo.gt.1) then 
        k=(khi+klo)/2 
        if(xa(k).gt.x)then 
          khi=k 
        else 
          klo=k 
        endif 
      goto 1 
      endif 
      h=xa(khi)-xa(klo) 
      if (h.eq.0.) pause 'bad xa input in splint' 
      a=(xa(khi)-x)/h 
      b=(x-xa(klo))/h 
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6. 
      return 
      end subroutine splint


	subroutine skewquad(ye,yt,angles,n_angles)
! =======================================================
! =======================================================
	implicit none
	integer i,n_angles
	real ye(n_angles),yt(n_angles),angles(n_angles)
	real poly1(3),poly2(3),pol1,pol2,sumpol1,sumpol2

	call fitquad(ye,angles,n_angles,poly1)
	call fitquad(yt,angles,n_angles,poly2)


! Divide expt by poly1
	sumpol1=0.0 	
	sumpol2=0.0 	

	do i=1,n_angles
	sumpol1=sumpol1+					&
	  poly1(1)+(poly1(2)*angles(i))+	&
	 (poly1(3)*(angles(i)**2))

	sumpol2=sumpol2+					&
	  poly2(1)+(poly2(2)*angles(i))+	&
	 (poly2(3)*(angles(i)**2))


	pol1=								&
	  poly1(1)+(poly1(2)*angles(i))+	&
	 (poly1(3)*(angles(i)**2))
	pol2=								&
	  poly2(1)+(poly2(2)*angles(i))+	&
	 (poly2(3)*(angles(i)**2))


	ye(i)=((ye(i)*pol2)/pol1)
	end do


	do i=1,n_angles
 	 ye(i)=(ye(i)*sumpol1)/sumpol2
	end do


	return
	end	subroutine skewquad	 



	SUBROUTINE FITQUAD(CURVE,ANGLES,N_ANGLES,B)

! ===================================================================
!     SUBROUTINE THAT WILL GENERATE A QUADRATIC	
!     This polynomial bends/skews the experimental data to the theory
!     data. 
!     This polynomial is estimated from the standard least squares 
!     formula except in this case
! ====================================================================
	
	implicit none
    INTEGER      I,INDX(3),N_ANGLES	
	REAL 	     CURVE(N_ANGLES),angles(n_angles)
	REAL         AB(3,3),B(3)
	REAL         x1,x2,x3,x4,y,xy,x2y
	REAL         DET



! Initialise Variables (just in case)

	x1  = 0.0
	x2  = 0.0
	x3  = 0.0
	x4  = 0.0
	xy  = 0.0
	x2y = 0.0
	y   = 0.0

	

	DO I=1,N_ANGLES
!	Calculate various sums needed for fit

	x1		= x1+angles(i)
	x2		= x2+(angles(i)**2)
	x3		= x3+(angles(i)**3)
	x4		= x4+(angles(i)**4)
	xy		= xy+(angles(i)*curve(i))
	x2y		= x2y+((angles(i)**2)*curve(i))
	y		= y+curve(i)

	END DO



! Set the AB Matrix Values

    AB(1,1) = real(n_angles)
	AB(1,2) = x1
	AB(1,3) = x2

	AB(2,1) = x1
	AB(2,2) = x2
	AB(2,3) = x3

	AB(3,1) = x2
	AB(3,2) = x3
	AB(3,3) = x4



! Set the B matrix Values

	b(1) = y
	b(2) = xy
	b(3) = x2y

		

	CALL LUDCMP(AB,3,3,INDX,DET)	
	CALL LUBKSB(AB,3,3,INDX,B)


	end subroutine fitquad



	subroutine rcs(curve,angles,n_angles)

!====================================================
! Applies Rutherford cross section
! Sin(theta/2)^4
!====================================================
	implicit none
	INTEGER	::	   I,N_ANGLES
	REAL 	::     CURVE(N_ANGLES),angles(n_angles)
	REAL    ::     PI,DEGTORAD,factor

	pi=4.0*atan(1.0)
	degtorad=pi/180.0

	 do i=1,n_angles
	  factor=(Sin(angles(i)*degtorad/2.0)**4)
	  curve(i)=curve(i)/factor
	 end do
	return
	end subroutine rcs



      SUBROUTINE lubksb(a,n,np,indx,b)
!===================================================
! Taken from Numerical recipies in f77
!===================================================
      INTEGER  :: n,np,indx(n)
      INTEGER  :: i,ii,j,ll
      REAL     :: a(np,np),b(n)
      REAL     :: sum
      ii=0
      do i=1,n
        ll=indx(i)
        sum=b(ll)
        b(ll)=b(i)
        if (ii.ne.0)then
          do j=ii,i-1
            sum=sum-a(i,j)*b(j)
          end do
        else if (sum.ne.0.) then
          ii=i
        endif
        b(i)=sum
      end do 
      do i=n,1,-1
        sum=b(i)
        do j=i+1,n
          sum=sum-a(i,j)*b(j)
        end do
        b(i)=sum/a(i,i)
	  end do	
      return
      end subroutine lubksb

      SUBROUTINE ludcmp(a,n,np,indx,d)
!===================================================
! Taken from Numerical recipies in f77
!===================================================

      INTEGER n,np,indx(n),NMAX
      INTEGER i,imax,j,k
      PARAMETER (NMAX=500,TINY=1.0e-20)
      REAL d,a(np,np),TINY
      REAL aamax,dum,sum,vv(NMAX)
      d=1.
      do 12 i=1,n
        aamax=0.
        do 11 j=1,n
          if (abs(a(i,j)).gt.aamax) aamax=abs(a(i,j))
11      continue
        if (aamax.eq.0.) pause 'singular matrix in ludcmp'
        vv(i)=1./aamax
12    continue
      do 19 j=1,n
        do 14 i=1,j-1
          sum=a(i,j)
          do 13 k=1,i-1
            sum=sum-a(i,k)*a(k,j)
13        continue
          a(i,j)=sum
14      continue
        aamax=0.
        do 16 i=j,n

          sum=a(i,j)
          do 15 k=1,j-1
            sum=sum-a(i,k)*a(k,j)
15        continue
          a(i,j)=sum
          dum=vv(i)*abs(sum)
          if (dum.ge.aamax) then
            imax=i
            aamax=dum
          endif
16      continue
        if (j.ne.imax)then
          do 17 k=1,n
            dum=a(imax,k)
            a(imax,k)=a(j,k)
            a(j,k)=dum
17        continue
          d=-d
          vv(imax)=vv(j)
        endif
        indx(j)=imax
        if(a(j,j).eq.0.)a(j,j)=TINY
        if(j.ne.n)then
          dum=1./a(j,j)

          do 18 i=j+1,n
            a(i,j)=a(i,j)*dum
18        continue
        endif
19    continue
      return
      end subroutine ludcmp





      SUBROUTINE group(natoms, u, numgr, ku, vu)

!********************************************************************************
! Groups the crystal
!********************************************************************************




        implicit none
		integer :: natoms,i,j
  		integer :: ku(natoms),ngr,number
		integer :: numgr, numat,k,iat, igr, perror, num
        real    :: u(natoms),vu(numgr+11),umin,umax
        real    :: utmp(maxnat), ktmp(maxnat)
        real	:: dist,crit,temp


        logical ::  SORTED

		crit=0.7
        num = natoms
        sorted = .False.
        k = 0

        Do i= 1,natoms
	        utmp(i) = u(i)
        END DO

! SORT LIST 
      Do While (.NOT.sorted)
      sorted = .True.
        k = k + 1
        Do j=1, num - k
        If (utmp(j) .GT. utmp(j + 1)) Then
          temp = utmp(j)
          utmp(j) = utmp(j + 1)
          utmp(j + 1) = temp
          sorted = .False.
        End If
	END DO
	END DO        

      
5   numgr = 1
    ktmp(1) = 1
        Do numat=2,natoms
        perror = 1
        Do i=1,(numat - 1)
            dist = Abs(utmp(numat) - utmp(i))
            If (dist .LT. crit) Then
              ktmp(numat) = ktmp(i)
            perror = 0
          End If
       END DO



   If (perror .GT.0) Then
        numgr = numgr + 1
        ktmp(numat) = numgr
        End If

	END DO



       do i=1,numgr
                do j=1,natoms
                        umin=utmp(1)
                        umax=utmp(1)
                        if(ktmp(j).eq.1) then
                                umin= min(umin,utmp(j))
                                umax =max(umax,utmp(j))
                        end if
                        if(abs(umax-umin).gt.crit) then
                                crit=0.9*crit
                                goto 5
                        end if
                end do

        end do
                               


         Do i=1,natoms
           Do k =1,natoms
          If (u(i) .EQ.  utmp(k)) Then
          ku(i) = ktmp(k)
          End If
		END DO
 	   END DO






       Do igr= 1,numgr
          number = 0
          vu(igr) = 0
          Do i=1,natoms
          If (ku(i) .EQ. igr) Then
            vu(igr) = vu(igr) + u(i)
            number = number + 1
            End If
		END DO

          If (number .GT. 1) Then
          vu(igr) = vu(igr) / number
          End If

       END DO



      Do iat=1,natoms

      u(iat) = u(iat) - vu(1)

	END DO

        If (numgr .GE. 2) Then
          igr = 2
          Do igr=2,numgr
            vu(igr) = vu(igr) - vu(1)
          END DO
        End If
        vu(1) = 0
        ngr = numgr



	end	subroutine group

	SUBROUTINE FITPOLY(CURVE,ANGLES,N_ANGLES,B)

! ===================================================================
!	SUBROUTINE THAT WILL GENERATE A 3RD ORDER SKEWING POLYNOMIAL	
! ====================================================================

    INTEGER      I,INDX(4),N_ANGLES	
	REAL 	     CURVE(N_ANGLES),angles(n_angles)
	REAL         AB(4,4),B(4)
	REAL         x1,x2,x3,x4,x5,x6,y,xy,x2y,x3y
	REAL         DET



! Initialise Variables (just in case)

	x1  = 0.0
	x2  = 0.0
	x3  = 0.0
	x4  = 0.0
	x5  = 0.0
	x6  = 0.0
	xy  = 0.0
	x2y = 0.0
	x3y = 0.0
	y   = 0.0

	





	DO I=1,N_ANGLES
!	Calculate various sums needed for fit

	x1		= x1+angles(i)
	x2		= x2+(angles(i)**2)
	x3		= x3+(angles(i)**3)
	x4		= x4+(angles(i)**4)
	x5		= x5+(angles(i)**5)
	x6		= x6+(angles(i)**6)
	xy		= xy+(angles(i)*curve(i))
	x2y		= x2y+((angles(i)**2)*curve(i))
	x3y		= x3y+((angles(i)**3)*curve(i))
	y		= y+curve(i)

	END DO



! Set the AB Matrix Values

      AB(1,1) = real(n_angles)
	AB(1,2) = x1
	AB(1,3) = x2
	AB(1,4) = x3
	AB(2,1) = x1
	AB(2,2) = x2
	AB(2,3) = x3
	AB(2,4) = x4
	AB(3,1) = x2
	AB(3,2) = x3
	AB(3,3) = x4
	AB(3,4) = x5
	AB(4,1) = x3
	AB(4,2) = x4
	AB(4,3) = x5
	AB(4,4) = x6 



! Set the B matrix Values

	b(1) = y
	b(2) = xy
	b(3) = x2y
	b(4) = x3y

		

	CALL LUDCMP(AB,4,4,INDX,DET)	
	CALL LUBKSB(AB,4,4,INDX,B)


	END subroutine fitpoly

	end module rfactor
	
